from tkinter import *
from tkinter import ttk
from tkinter import Tk
from tkinter import filedialog
from pathlib import Path
from PIL import Image
import qrcode
import os

OPTION_IMAGE_FORMAT = ['.png', '.jpg', '.jpeg', '.gif', '.webp', '.ico', '.tif', '.tiff']
QR_FILL_COLOR_DEFAULT = '#000'
QR_BACK_COLOR_DEFAULT = '#fff'
QR_BOX_SIZE_DEFAULT = 150
QR_BOX_SIZE_MAX = 300
QR_BOX_SIZE_MIN = 1
QR_BORDER_SIZE_DEFAULT = 2
QR_BORDER_SIZE_MAX = 40
QR_BORDER_SIZE_MIN = 1
savePathSelected = False
override = False
customizationResultSumMsg = str()

def selectSavePath():
    global savePath
    global savePathSelected
    savePath = filedialog.askdirectory()
    if not savePath == "":
        savePathSelected = True

def addImage():
    global imgPath
    global img
    imgPath = filedialog.askopenfilename()
    try:
        img = Image.open(imgPath).convert('RGBA')
        selectedImg.set(os.path.basename(imgPath))
        printCmResultMsg("")
        checkCustomization()
    except:
        if imgPath == "":
            printCmResultMsg("")
            checkCustomization()
        else: printCmResultMsg("Not an image")

def printResultMsg(msg, color):
    global resultMsgLabel
    resultMsgLabel = Label(outputframe, textvariable=resultMsg, fg=color).grid(column=1, row=6)
    resultMsg.set(msg)
    
    # Sets customizationResultMsg
def printCmResultMsg(msg):
    global customizationResultMsg
    global customizationResultSumMsg
    NOTE_MSG = "\nNote: If an customizer error occurs the default value will be used."
    if msg == "":
        customizationResultSumMsg = ""
    else:
        customizationResultSumMsg += f"- {msg}\n"
    if msg != "":
        customizationResultMsg.set(customizationResultSumMsg + NOTE_MSG)
    else: customizationResultMsg.set("")


def generate_QR_Code(data, imgName):
    global savePath
    global savePathSelected
    global img
    format = optionValue.get()
    printResultMsg("", "#000")

    checkCustomization()

    if not savePathSelected or savePath == "":
        savePath = str(Path.home() / "Downloads")
        savePathSelected = True
    if data == "":
        printResultMsg("Please enter some Data", "#f00")
    elif imgName == "":
        printResultMsg("Please enter an Image Name", "#f00")
    else:
        qr = qrcode.QRCode(
            version = 1,
            error_correction=qrcode.constants.ERROR_CORRECT_L,
            box_size = qrBoxSize,
            border = qrBorderSize
        )
        qr.add_data(data)
        qr.make()
        qr = qr.make_image(
            fill_color = qrFillColor,
            back_color = qrBackColor
        )

        try:
            img = img.resize([int(qr.size[0]*0.15), int(qr.size[1]*0.15)])
            pos = ((qr.size[0] - img.size[0]) // 2, (qr.size[1] - img.size[1]) // 2)
            qr.paste(img, pos)
        except NameError:
            print("Skipped image paste due to no image found/selected")

        if os.path.exists(f"{savePath}/{imgName}{format}"):
            agreeOverride()
            root.wait_window(overrideWindow)
            if(override):
                qr.save(f"{savePath}/{str(imgName) + str(format)}")
                printResultMsg(f"QR Code generated and saved as {imgName}{format}", "#0a0")
            else: return
        else:
            qr.save(f"{savePath}/{str(imgName) + str(format)}")
            printResultMsg(f"QR Code generated and saved as {imgName}{format}", "#0a0")


def agreeOverride():

    def cancle():
        global override
        override = False
        overrideWindow.destroy()

    def overrideF():
        global override
        override = True
        overrideWindow.destroy()

    global overrideWindow
    overrideWindow = Toplevel()
    overrideWindow.title("")
    overrideWindow.grab_set()

    overrideframe = ttk.Frame(overrideWindow, padding="20 20 20 20")
    overrideframe.grid(column=0, row=0)

    orbuttonframe = ttk.Frame(overrideframe)
    orbuttonframe.grid(column=1, row=2)

    ttk.Label(overrideframe, text="File already exists. Wanna override it?").grid(column=1, row=1)
    ttk.Button(orbuttonframe, text="cancel", command=cancle).grid(column=1, row=2)
    ttk.Button(orbuttonframe, text="override", command=overrideF).grid(column=2, row=2)

def checkCustomization():
    printCmResultMsg("")

    global qrFillColor
    global qrBackColor
    global qrBoxSize
    global qrBorderSize
    qrFillColor = qrFillColor_entry.get()
    qrBackColor = qrBackColor_entry.get()
    try: qrBoxSize = qrSize_entry.get()
    except:
        printCmResultMsg("Invalid size!")
        qrBoxSize = QR_BOX_SIZE_DEFAULT
    try: qrBorderSize = qrBorder_entry.get()
    except:
        printCmResultMsg("Invalid border size!")
        qrBorderSize = QR_BORDER_SIZE_DEFAULT

    # Set fallback if color invalid
    if qrFillColor == '':
        qrFillColor = QR_FILL_COLOR_DEFAULT
    elif not checkHexCode(qrFillColor):
        qrFillColor = QR_FILL_COLOR_DEFAULT
        printCmResultMsg("Color only supports Hexcodes!")

    if qrBackColor == '':
        qrBackColor = QR_BACK_COLOR_DEFAULT
    elif not checkHexCode(qrBackColor):
        qrBackColor = QR_BACK_COLOR_DEFAULT
        printCmResultMsg("Background color only supports Hexcodes!")

    # Set fallback if boxsize invalid
    if not str(qrBoxSize).isnumeric():
        qrBoxSize = QR_BOX_SIZE_DEFAULT
    if qrBoxSize < QR_BOX_SIZE_MIN or qrBoxSize > QR_BOX_SIZE_MAX:
        qrBoxSize = QR_BOX_SIZE_DEFAULT
        printCmResultMsg("Size out of range!")

    # Set fallback if border invalid
    if not str(qrBorderSize).isnumeric():
        qrBorderSize = QR_BORDER_SIZE_DEFAULT
    if qrBorderSize < QR_BORDER_SIZE_MIN or qrBorderSize > QR_BORDER_SIZE_MAX:
        qrBorderSize = QR_BORDER_SIZE_DEFAULT
        printCmResultMsg("Border Size out of range!")


def checkHexCode(code):
    HEX_CODE_CHARS = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F']
    if code[0] == '#':
        if len(code) == 4 or len(code) == 7:
            for char in code[1:]:
                if char.upper() not in HEX_CODE_CHARS:
                    return False
            return True
    return False

def callbackInput(var, index, mode):
    checkCustomization()

root = Tk()
root.title("QR Code Generator")
root.grid_rowconfigure(0, weight=1)
root.grid_columnconfigure(0, weight=1)

mainframe = ttk.Frame(root, padding="20 20 20 20")
mainframe.grid(column=0, row=0, sticky='')
mainframe.grid_columnconfigure(0, weight=1)
mainframe.grid_rowconfigure(0, weight=1)

buttonframe = ttk.Frame(mainframe)
buttonframe.grid(column=1, row=5)

outputframe = ttk.Frame(mainframe)
outputframe.grid(column=1, row=6)

customizerframe = ttk.Frame(mainframe)
customizerframe.grid(column=1, row=7, pady=10)

# --- ROW 1 ---
ttk.Label(mainframe, text="Text or URL to convert:").grid(column=1, row=1, sticky='w')

# --- ROW 2 ---
dataInput = StringVar()
ttk.Entry(mainframe, textvariable=dataInput,  width=45).grid(column=1, row=2)

# --- ROW 3 ---
ttk.Label(mainframe, text="Image Name:").grid(column=1, row=3, sticky='w')

# --- ROW 4 ---
imgName = StringVar()
ttk.Entry(mainframe, textvariable=imgName, width=45).grid(column=1, row=4)

# --- ROW 5 ---
ttk.Button(buttonframe, text="Save as...", command=selectSavePath).grid(column=1, row=5, pady=(10, 0))
optionValue = StringVar(root)
optionValue.set(OPTION_IMAGE_FORMAT[0])
ttk.OptionMenu(buttonframe, optionValue, OPTION_IMAGE_FORMAT[0], *OPTION_IMAGE_FORMAT).grid(column=2, row=5, padx=5, pady=(10, 0))
genBtn = ttk.Button(buttonframe, text="Generate QR Code", command= lambda: generate_QR_Code(dataInput.get(), imgName.get())).grid(column=3, row=5, padx=5, pady=(10, 0))

# --- ROW 6 ---
resultMsg = StringVar()
resultMsgLabel = Label(outputframe, textvariable=resultMsg, fg="#0a0").grid(column=1, row=6)

# --- ROW 7 ---
ttk.Label(customizerframe, text="Fill color:").grid(column=1, row=7, sticky='w')
ttk.Label(customizerframe, text="Background color: ").grid(column=2, row=7, sticky='w', padx=15)

# --- ROW 8 ---
qrFillColor_entry = StringVar()
qrFillColor_entry.set(QR_FILL_COLOR_DEFAULT)
qrFillColor_entry.trace_add('write', callbackInput)
ttk.Entry(customizerframe, textvariable=qrFillColor_entry).grid(column=1, row=8, sticky='w')
qrBackColor_entry = StringVar()
qrBackColor_entry.set(QR_BACK_COLOR_DEFAULT)
qrBackColor_entry.trace_add('write', callbackInput)
ttk.Entry(customizerframe, textvariable=qrBackColor_entry).grid(column=2, row=8, sticky='w', padx=15)

# --- ROW 9 ---
ttk.Label(customizerframe, text=f"Size ({QR_BOX_SIZE_MIN}-{QR_BOX_SIZE_MAX}):").grid(column=1, row=9, sticky='w')
ttk.Label(customizerframe, text=f"Border Size ({QR_BORDER_SIZE_MIN}-{QR_BORDER_SIZE_MAX}):").grid(column=2, row=9, sticky='w', padx=15)

# --- ROW 10 ---
qrSize_entry = IntVar()
qrSize_entry.set(QR_BOX_SIZE_DEFAULT)
qrSize_entry.trace_add('write', callbackInput)
ttk.Entry(customizerframe, textvariable=qrSize_entry).grid(column=1, row=10, sticky='w')
qrBorder_entry = IntVar()
qrBorder_entry.set(QR_BORDER_SIZE_DEFAULT)
qrBorder_entry.trace_add('write', callbackInput)
ttk.Entry(customizerframe, textvariable=qrBorder_entry).grid(column=2, row=10, sticky='w', padx=15)

# --- ROW 11 ---
ttk.Button(customizerframe, text="Add image", command=addImage).grid(column=1, row=12, columnspan=2, sticky='s')
selectedImg = StringVar()
ttk.Label(customizerframe, textvariable=selectedImg, wraplength=100).grid(column=1, row=13, columnspan=2, sticky='n')
ttk.Label(customizerframe, text="(Beta)").grid(column=1, row=14, columnspan=2)

customizationResultMsg = StringVar()
customizationResultMsg.set("")
ttk.Label(customizerframe, textvariable=customizationResultMsg, foreground="#fc9003", wraplength=250).grid(column=1, row=15, columnspan=2, pady=(20, 0))

root.mainloop()
